import sys
from protocol import IcingProtocol

RED = "\033[91m"
GREEN = "\033[92m"
YELLOW = "\033[93m"
BLUE = "\033[94m"
RESET = "\033[0m"


def main():
    protocol = IcingProtocol()

    print(f"{YELLOW}\n======================================")
    print("   Icing Protocol - Manual CLI Demo   ")
    print("======================================\n" + RESET)
    print(f"Listening on port: {protocol.local_port}")
    print(f"Your identity public key (hex): {protocol.identity_pubkey.hex()}")
    print("\nAvailable commands:")
    print("  peer_id <hex_pubkey>")
    print("  connect <port>")
    print("  generate_ephemeral_keys")
    print("  send_ping")
    print("  send_handshake")
    print("  respond_ping <index> <0|1>")
    print("  generate_ecdhe <index>")
    print("  derive_hkdf")
    print("  send_encrypted <plaintext>")
    print("  decrypt_received <index>")
    print("  auto_responder <on|off>")
    print("  show_state")
    print("  exit\n")

    while True:
        try:
            line = input("Cmd> ").strip()
        except EOFError:
            break
        if not line:
            continue
        parts = line.split()
        cmd = parts[0].lower()
        if cmd == "exit":
            protocol.stop()
            break
        elif cmd == "show_state":
            protocol.show_state()
        elif cmd == "peer_id":
            if len(parts) != 2:
                print("Usage: peer_id <hex_pubkey>")
                continue
            protocol.set_peer_identity(parts[1])
        elif cmd == "connect":
            if len(parts) != 2:
                print("Usage: connect <port>")
                continue
            try:
                port = int(parts[1])
                protocol.connect_to_peer(port)
            except ValueError:
                print("Invalid port.")
        elif cmd == "generate_ephemeral_keys":
            protocol.generate_ephemeral_keys()
        elif cmd == "send_ping":
            protocol.send_ping_request()
        elif cmd == "send_handshake":
            protocol.send_handshake()
        elif cmd == "respond_ping":
            if len(parts) != 3:
                print("Usage: respond_ping <index> <0|1>")
                continue
            try:
                idx = int(parts[1])
                ac = int(parts[2])
                protocol.respond_to_ping(idx, ac)
            except ValueError:
                print("Index and answer must be integers.")
        elif cmd == "generate_ecdhe":
            if len(parts) != 2:
                print("Usage: generate_ecdhe <index>")
                continue
            try:
                idx = int(parts[1])
                protocol.generate_ecdhe(idx)
            except ValueError:
                print("Index must be an integer.")
        elif cmd == "derive_hkdf":
            protocol.derive_hkdf()
        elif cmd == "send_encrypted":
            if len(parts) < 2:
                print("Usage: send_encrypted <plaintext>")
                continue
            plaintext = " ".join(parts[1:])
            protocol.send_encrypted_message(plaintext)
        elif cmd == "decrypt_received":
            if len(parts) != 2:
                print("Usage: decrypt_received <index>")
                continue
            try:
                idx = int(parts[1])
                protocol.decrypt_received_message(idx)
            except ValueError:
                print("Index must be an integer.")
        elif cmd == "auto_responder":
            if len(parts) != 2:
                print("Usage: auto_responder <on|off>")
                continue
            protocol.enable_auto_responder(parts[1].lower() == "on")
        else:
            print(f"{RED}[ERROR]{RESET} Unknown command: {cmd}")

if __name__ == "__main__":
    main()
